#/usr/bin/!perl

use Test::More qw(no_plan);

use_ok( 'AI::Genetic::Pro' );
use_ok( 'Class::Accessor::Fast::XS' );
use_ok( 'Clone' );
use_ok( 'GD::Graph::linespoints' );
use_ok( 'Digest::MD5' );
use_ok( 'Exporter::Lite' );	
use_ok( 'List::MoreUtils' );
use_ok( 'List::Util' );
use_ok( 'Math::Random' );
use_ok( 'Struct::Compare' );
use_ok( 'Tie::Array::Packed' );
use_ok( 'UNIVERSAL::require' );
